#include <cstdint>
#include <glad/glad.h>
#include <GLFW/glfw3.h>
#include <iostream>

#include <coil/coil.hpp>
#include <coil/segment.hpp>
#include <utils/utils.hpp>

Coil::Coil(uint32_t currentYear, uint32_t amountOfSegments, uint32_t sliceDetalization, float radius, float width) 
	: radius(radius), width(width), currentYear(currentYear) {
	for (short i = amountOfSegments / -2.; i < amountOfSegments / 2.; i++) {
		uint32_t year = this->currentYear + i;

    	segments.push_back(CoilSegment(this, year, segmentHeight, i, sliceDetalization,false));
	}
};

std::vector<CoilSegment> &Coil::getSegments() {
	return segments;
}

float Coil::getWidth() {
	return width;
}

float Coil::getRadius() {
	return radius;
}

void Coil::initGLBuffers() {

	glGenVertexArrays(1, &VAO);
	glGenBuffers(1, &VBO);
	glGenBuffers(1, &EBO);

	glBindVertexArray(VAO);

	glBindBuffer(GL_ARRAY_BUFFER, VBO);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, EBO);

	// glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(indices), indices, GL_STATIC_DRAW);

	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 6 * sizeof(float), (void*)0);
	glEnableVertexAttribArray(0);

	glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, 6 * sizeof(float), (void*) (3 * sizeof(float)));
	glEnableVertexAttribArray(1);

	glBindVertexArray(0);
}

void Coil::render() {
	glBindVertexArray(VAO);
	std::cout << "Rendering a coil." << std::endl;
	for (CoilSegment segment : segments) {
		std::cout << "Rendering " << segment.getYear() << std::endl;
		segment.fillVBO();
		uint64_t indicesCount = segment.fillEBO(EBO);
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, EBO);
		// glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		// glDrawArrays(GL_POINTS, 0, segment.getVerticesLength());
		glDrawElements(GL_TRIANGLES_ADJACENCY, sizeof(unsigned int)*indicesCount, GL_UNSIGNED_INT, 0);
	}
	glBindVertexArray(0);
}